/*
  $Id: config.c,v 1.2 2000/03/02 09:03:35 jacobs Exp jacobs $

  $Log: config.c,v $
  Revision 1.2  2000/03/02 09:03:35  jacobs
  3 new filters (MAX_PAUSE, SG_PLAYERDETAILS, PLAYERS_MINIMUM)

  Revision 1.1  2000/02/14 11:02:13  jacobs
  Initial revision

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "tools/hpj_tfile.h"

#ifndef __file__
#define __file__ "config.c"
#endif


char cfg_apth[NOF_FORMATS][3]={
  "Na", "Ti", "Ga", "Gw", "Ki", "Su", "Tk", "De", "Fr", "Fh",\
  "Ef", "Tt", "It", "Ta", "So",\
  "Cs", "Fs", "Fp", "Fl", "Fu", "Fa", "Fc", "Fk", "Fe", "Cg", "Cw"};


/* constructor :-) */
hpj_config *config_new()
{
  hpj_config *cfg=(hpj_config *)malloc(sizeof(hpj_config));
  if (cfg==NULL)
    {
      fprintf(stderr, "  %s : config_new : MALLOC-Failure\n", __file__);
      exit(-1);
    }
  memset(cfg, 0, sizeof(hpj_config));
  cfg->b_ltstats=1;
  return cfg;
}


/* another consturctor */
hpj_config *config_new_load(char *filename)
{
  hpj_config *cfg=config_new();
  hpj_tfile  *f=tfile_new(filename);
  hpj_string *s, *k;
  CHAR        b_done, *c1, *c2;
  int         hour, min, sec;

  if (tfile_open(f)==false)
    {
      f=tfile_del(f);
      fprintf(stderr, "  %s : config_new_load : FILE-Error\n",__file__);
      return cfg;
    }

  while(!tfile_eof(f))
    {
      s=tfile_getLine(f);
      if (string_length(s)<1 || string_char(s,0)=='#' || string_char(s,0)=='/')
	{
	  s=string_del(s);
	  continue;
	}
      k=string_token_left(s, ' ');
      string_shiftLeft(s, string_length(k));
      while(string_char(s,0) == ' ')
	string_shiftLeft(s,1);

      /* now we have in k the key, in s the var */
      //fprintf(stderr, "%s --> *%s*\n", string_charptr(k), string_charptr(s));
      b_done=0;
      c1=string_charptr(k);
      c2=string_charptr(s);
      if (strcmp(c1, "USE_ALLGAMES")==0)
	{  cfg->b_showall=atoi(c2); b_done=1;}

      if (strcmp(c1, "USE_FUNNAMES")==0)
	{  cfg->b_funname=atoi(c2); b_done=1;}

      if (strcmp(c1, "SG_PLAYERDETAILS")==0)
	{  cfg->b_sgpdetails=atoi(c2); b_done=1;}

      if (strcmp(c1, "SG_TALKS")==0)
	{  cfg->b_sgtalks=atoi(c2);b_done=1;}

      if (strcmp(c1, "USE_LONGTERM")==0)
	{  cfg->b_ltstats=atoi(c2); b_done=1;}

      if (strcmp(c1, "LONGTERM_FILE")==0)
	{ cfg->lt_file=strdup(c2); b_done=1;}

      if (strcmp(c1, "WRITE_PURETEXT")==0)
	{  cfg->b_puretext=atoi(c2);b_done=1;}

      if (strcmp(c1, "USE_WARMUPS")==0)
	{  cfg->b_warmup=atoi(c2);b_done=1;}

      if (strcmp(c1, "MINIMUM_PLAYERS")==0)
	{  cfg->player_minimum=atoi(c2);b_done=1;}

      if (strcmp(c1, "MINIMUM_GAMES")==0)
	{  cfg->game_minimum=atoi(c2);b_done=1;}

      if (strcmp(c1, "MAXIMUM_PAUSE")==0)
	{  cfg->pause_maximum=atoi(c2);b_done=1;}

      if (strcmp(c1, "MINIMUM_LTTIME")==0)
	{
	  sscanf(c2, "%02i:%02i:%02i", &hour, &min, &sec);
	  cfg->time_ltminimum=hour*3600 + min*60 + sec;
	  b_done=1;
	}

      if (strcmp(c1, "MINIMUM_TIME")==0)
	{
	  sscanf(c2, "%02i:%02i:%02i", &hour, &min, &sec);
	  cfg->time_minimum=hour*3600 + min*60 + sec;
	  b_done=1;
	}

      if (strcmp(c1, "KICK_LAMERS")==0)
	{  cfg->b_kick=atoi(c2);b_done=1;}

      if (strcmp(c1, "LINE_LIMIT")==0)
	{  cfg->linelimit=atoi(c2);b_done=1;}

      if (strcmp(c1, "PNG_COMPRESSION")==0)
	{  cfg->zcompression=atoi(c2);b_done=1;}

      if (strcmp(c1, "OUTPUTDIR")==0)
	{  cfg->outdir=strdup(c2);b_done=1;}
      if (strcmp(c1, "TEMPLATEDIR")==0)
	{  cfg->templdir=strdup(c2);b_done=1;}
      if (strcmp(c1, "LOADMAPFILE")==0)
	{  cfg->mapfile=strdup(c2);b_done=1;}
      if (strcmp(c1, "CALCULATION")==0)
	{  cfg->calculation=strdup(c2);b_done=1;}
      
      if (strcmp(c1, "AP_TABLEFORMAT")==0)
	{
	  if (cfg->tableformat!=NULL)
	    cfg->tableformat=iarray_del(cfg->tableformat);
	  cfg->tableformat=config_parseTableFormat(c2);
	  b_done=1;
	}

      if (strcmp(c1, "AP_FORCEMYFORMAT")==0)
	{  cfg->b_forcemyformat=atoi(c2);b_done=1;}

      if (!b_done)
	fprintf(stderr, "Unparseable config-line : %s\n", c1);
      s=string_del(s);
      k=string_del(k);
    }

  tfile_close(f);
  f=tfile_del(f);
  return cfg;
}





hpj_iarray *config_parseTableFormat(char *s)
{
  hpj_iarray *tf;
  char        format[3];
  int         i, j;
  
  if (s==NULL)
    return NULL;

  tf=iarray_new(2);
  format[2]=0;
  for (i=0; i<strlen(s); i++)
    {
      format[0]=s[i  ];
      format[1]=s[i+1];
      i++;
      for (j=0; j<NOF_FORMATS; j++)
	{
	  if (strcmp(format, cfg_apth[j])==0)
	    {
	      iarray_append(tf, j);
	      break;
	    }
	}
      if (j>=NOF_FORMATS)
	fprintf(stderr, "FORMAT-String-Error : %s\n", format);
    }
  return tf;
}




void config_show(hpj_config *cfg)
{
  printf(" use allgames  : %i\n", cfg->b_showall);
  printf(" use funnames  : %i\n", cfg->b_funname);
  printf(" sgp-details   : %i\n", cfg->b_sgpdetails);
  printf(" sg talks      : %i\n", cfg->b_sgtalks);
  printf(" use longterms : %i\n", cfg->b_ltstats);
  printf(" use warmups   : %i\n", cfg->b_warmup);
  printf(" write puretext: %i\n", cfg->b_puretext);
  printf(" kick lamers   : %i\n", cfg->b_kick);
  printf(" min. players  : %i\n", cfg->player_minimum);
  printf(" line-limit    : %i\n", cfg->linelimit);
  printf(" game-minim.   : %i\n", cfg->game_minimum);
  printf(" pause maximum : %i\n", cfg->pause_maximum);
  printf(" force myformat: %i\n", cfg->b_forcemyformat);
  printf(" lt_time-min.  : %i sec\n", cfg->time_ltminimum);
  printf(" game_time-min.: %i sec\n", cfg->time_minimum);
  printf(" png-compress. : %i\n", cfg->zcompression);
  printf(" outputdir     : %s\n", cfg->outdir);
  printf(" template-dir  : %s\n", cfg->templdir);
  printf(" mapfile       : %s\n", cfg->mapfile);
  printf(" calculation   : %s\n", cfg->calculation);
}



hpj_config *config_del(hpj_config *cfg)
{
  if (cfg==NULL)
    {
      fprintf(stderr, "  %s : config_del : NULL-Ptr\n", __file__);
      return NULL;
    }
  if (cfg->templdir) free(cfg->templdir);
  if (cfg->outdir) free(cfg->outdir);
  if (cfg->calculation) free(cfg->calculation);
  if (cfg->mapfile) free(cfg->mapfile);
  if (cfg->lt_file) free(cfg->lt_file);
  if (cfg->tableformat!=NULL)
    cfg->tableformat=iarray_del(cfg->tableformat);
  if (cfg->templateformat!=NULL)
    cfg->templateformat=iarray_del(cfg->templateformat);
  free(cfg);
  return NULL;
}
