/*
  Implementation of the long-time statistics.

  $Id: lt_stats.c,v 1.10 2000/03/02 09:03:35 jacobs Exp jacobs $

  $Log: lt_stats.c,v $
  Revision 1.10  2000/03/02 09:03:35  jacobs
  fixed player-name==NULL - bug

  Revision 1.9  2000/02/14 11:02:13  jacobs
  *** empty log message ***

  Revision 1.8  2000/02/06 20:18:43  jacobs
  *** empty log message ***

  Revision 1.7  2000/01/29 21:41:12  jacobs
  *** empty log message ***

  Revision 1.6  2000/01/14 21:49:15  jacobs
  *** empty log message ***

  Revision 1.5  2000/01/11 23:25:53  jacobs
  *** empty log message ***

  Revision 1.4  2000/01/09 02:32:17  jacobs
  *** empty log message ***

  Revision 1.3  2000/01/03 16:04:02  jacobs
  *** empty log message ***

  Revision 1.2  1999/12/31 02:21:41  jacobs
  *** empty log message ***

  Revision 1.1  1999/12/30 01:26:21  jacobs
  Initial revision


*/

#ifndef __file__
#define __file__ "lt_stats.c"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lt_stats.h"
#include "tool.h"

char savestats_version='4';
char loadstats_version;

/*
 *
 *
 Functions for handling lt_players
 *
 *
 */
t_ltp *new_ltPlayer(void)
{
  t_ltp *ltp=(t_ltp *)malloc(sizeof(t_ltp));
  if (ltp==NULL)
    {
      fprintf(stderr, "  %s : new_ltPlayer : MALLOC-Failure\n", __file__); 
      return NULL;
    }
  memset (ltp, 0, sizeof(t_ltp));
  return ltp;
}


t_ltp *del_ltPlayer(t_lt_player *ltp)
{
  if (ltp==NULL) return NULL;
  if (ltp->name) free(ltp->name);
  free(ltp);
  return NULL;
}

long int size_ltPlayer(t_lt_player *ltp)
{
  long int sum=sizeof(t_lt_player);
  sum+=strlen(ltp->name)+1;
  return sum;
}

int save_ltPlayer(t_lt_player *ltp, FILE *fp)
{
  long int  size=sizeof(long int), i, k;

  /* magic bytes */
  fprintf(fp, "ltp");

  /* name of the player */
  k=strlen(ltp->name);
  fwrite(&k,size,1,fp);
  fwrite(ltp->name,k,1,fp);

  fwrite(&ltp->secs,size,1,fp);
  fwrite(&ltp->skill,size,1,fp);
  fwrite(&ltp->pid,size,1,fp);

  fwrite(&ltp->nof_frags,size,1,fp);
  fwrite(&ltp->nof_kills,size,1,fp);
  fwrite(&ltp->nof_suicides,size,1,fp);
  fwrite(&ltp->nof_tkills,size,1,fp);
  fwrite(&ltp->nof_deaths,size,1,fp);
  fwrite(&ltp->nof_says,size,1,fp);
  fwrite(&ltp->nof_items,size,1,fp);
  fwrite(&ltp->nof_games,size,1,fp);
  fwrite(&ltp->nof_wins,size,1,fp); // since version 2
  fwrite(&ltp->nonactive, size,1,fp); //since version 4

  // ctf was introduced with version 3
  fwrite(&ltp->nof_ctfgames, size,1,fp);
  fwrite(&ltp->nof_ctfwins, size,1,fp);
  fwrite(&ltp->ctf_score, size,1,fp);
  fwrite(&ltp->ctf_steals, size,1,fp);
  fwrite(&ltp->ctf_pickups, size,1,fp);
  fwrite(&ltp->ctf_loss, size,1,fp);
  fwrite(&ltp->ctf_suic, size,1,fp);
  fwrite(&ltp->ctf_autoret, size,1,fp);
  fwrite(&ltp->ctf_captures, size,1,fp);
  fwrite(&ltp->ctf_kills, size,1,fp);
  fwrite(&ltp->ctf_returns, size,1,fp);

  /* now the items */
  k=NOF_ITEMS;
  fwrite(&k,size,1,fp);
  for(i=0; i<k; i++)
    fwrite(&ltp->items[i],size,1,fp);

  /* now the weapons
     WE JUST WRITE THE NOF_WEAPS ONCE !!!
  */
  k=NOF_WEAPS;
  fwrite(&k,size,1,fp);
  for(i=0; i<k; i++)
    {
      fwrite(&ltp->weaps[i],size,1,fp);
      fwrite(&ltp->wsui[i],size,1,fp);
      fwrite(&ltp->wtkill[i],size,1,fp);
      fwrite(&ltp->wdeath[i],size,1,fp);
    }
  return 0;
}


int load_ltPlayer(t_lt_player *ltp, FILE *fp, hpj_cstack *s1)
{
  long int  size=sizeof(long int), i, k;
  char      magic[6];

  fread(magic, 3,1,fp);
  if (strncmp(magic,"ltp",3)!=0)
     {
      fprintf(stderr, "  lt-stat - file is corrupted\n");
      return 0;
    }

  /* name of the player */
  fread(&k,size,1,fp);
  ltp->name=(char *)malloc(sizeof(char) * (k+1));
  fread(ltp->name,k,1,fp);
  ltp->name[k]=0;

  fread(&ltp->secs,size,1,fp);
  fread(&ltp->skill,size,1,fp);
  fread(&ltp->pid,size,1,fp);

  fread(&ltp->nof_frags,size,1,fp);
  fread(&ltp->nof_kills,size,1,fp);
  fread(&ltp->nof_suicides,size,1,fp);
  fread(&ltp->nof_tkills,size,1,fp);
  fread(&ltp->nof_deaths,size,1,fp);
  fread(&ltp->nof_says,size,1,fp);
  fread(&ltp->nof_items,size,1,fp);
  fread(&ltp->nof_games,size,1,fp);
  if (loadstats_version!='1')
    fread(&ltp->nof_wins,size,1,fp);
  if (loadstats_version>'3')
    fread(&ltp->nonactive, size,1,fp);
  if (loadstats_version>'2')
    {
      fread(&ltp->nof_ctfgames, size,1,fp);
      fread(&ltp->nof_ctfwins, size,1,fp);
      fread(&ltp->ctf_score, size,1,fp);
      fread(&ltp->ctf_steals, size,1,fp);
      fread(&ltp->ctf_pickups, size,1,fp);
      fread(&ltp->ctf_loss, size,1,fp);
      fread(&ltp->ctf_suic, size,1,fp);
      fread(&ltp->ctf_autoret, size,1,fp);
      fread(&ltp->ctf_captures, size,1,fp);
      fread(&ltp->ctf_kills, size,1,fp);
      fread(&ltp->ctf_returns, size,1,fp);
    }

  /* now the items */
  fread(&k,size,1,fp);
  for(i=0; i<k; i++)
    fread(&ltp->items[i],size,1,fp);

  /* now the weapons
     WE JUST WRITE THE NOF_WEAPS ONCE !!!
     Weapons are written as triple !!!
  */
  fread(&k,size,1,fp);
  for(i=0; i<k; i++)
    {
      fread(&ltp->weaps[i],size,1,fp);
      fread(&ltp->wsui[i],size,1,fp);
      fread(&ltp->wtkill[i],size,1,fp);
      fread(&ltp->wdeath[i],size,1,fp);
    }

  if (loadstats_version=='1')
    {
      size=sizeof(double);
      fread(&ltp->effect,size,1,fp);
      fread(&ltp->ttl,size,1,fp);
      fread(&ltp->fph,size,1,fp);
    }

  ltp->ttl=(double)ltp->secs/(double)(ltp->nof_deaths+1);
  ltp->effect=ltp_effectiveness(ltp, s1);
  if (ltp->secs)
    ltp->fph=(3600.0/(double)ltp->secs) * ltp->nof_frags;

  return 0;
}















/*
 *
 *
 BASIC functions for lt_stats
 *
 *
 */
t_lts *new_ltStat(void)
{
  t_lts *lts=(t_lts *)malloc(sizeof(t_lts));
  if (lts==NULL)
    {
      fprintf(stderr, "  %s : new_ltStat : MALLOC-Failure\n", __file__); 
      return NULL;
    }
  memset(lts, 0, sizeof(t_lts));
  lts->ltplayers=(t_ltp **)malloc(sizeof(t_ltp *) * MAX_LT_PLAYERS);
  if (lts->ltplayers==NULL)
    {
      fprintf(stderr, "  %s : new_ltStat : MALLOC-Failure\n", __file__); 
      return NULL;
    }
  return lts;
}

t_lts *del_ltStat(t_lts *lts)
{
  int i;

  if (lts==NULL) return NULL;

  for (i=0; i<lts->nof_players; i++)
    del_ltPlayer(lts->ltplayers[i]);
  if (lts->ltplayers) free(lts->ltplayers);
  free(lts);
  return NULL;
}

long int size_ltStat(t_lts *lts)
{
  long int sum, i;
  sum=sizeof(t_lts);
  for(i=0; i<lts->nof_players; i++)
    sum+=size_ltPlayer(lts->ltplayers[i]);
  return sum;
}


/*
 *
 *
 "higher" functions for the lt_stats
 *
 *
 */
int add_game(t_lts *lts, t_game *game, hpj_cstack *s1)
{
  int    i, j;
  char   known;
  t_ltp *ltp;

  if (lts==NULL || game==NULL) return -1;

  lts->nof_games++;
  lts->nof_kills+=game->nof_kills;
  lts->nof_tkills+=game->nof_tkills;
  lts->nof_deaths+=game->nof_deaths;
  lts->nof_items+=game->nof_items;
  lts->nof_suicides+=game->nof_suicides;  
  lts->nof_frags+=game->nof_frags;
  lts->secs+=game->secs;
  lts->nof_says+=game->nof_says;

  if (game->type == GAMETYPE_CTF)
    lts->nof_ctfgames++;

  for(j=0; j<lts->nof_players; j++)
    lts->ltplayers[j]->nonactive++;

  /* for each player we look for a lt_player with the same name */
  for (i=0; i<game->nof_players; i++)
    {
      known=0;
      if (game->player[i]->name==NULL) continue;
      for(j=0; j<lts->nof_players; j++)
	if (strcmp(game->player[i]->name, lts->ltplayers[j]->name)==0)
	  {known=1;lts->ltplayers[j]->nonactive=0;break;}

      if (!known)
	{
	  lts->ltplayers[lts->nof_players]=new_ltPlayer();
	  lts->ltplayers[lts->nof_players]->name=strdup(game->player[i]->name);
	  lts->ltplayers[lts->nof_players]->pid=lts->nof_players;
	  lts->nof_players++;
	}

      ltp=lts->ltplayers[j];

      if (game->player[i]->b_winner)
	ltp->nof_wins++;
      ltp->nof_games++;
      ltp->skill=game->player[i]->skill;
      ltp->nof_kills+=game->player[i]->nof_kills;
      ltp->nof_suicides+=game->player[i]->nof_suicides;
      ltp->nof_tkills+=game->player[i]->nof_tkills;
      ltp->nof_frags+=game->player[i]->nof_frags;
      ltp->nof_deaths+=game->player[i]->nof_deaths;
      ltp->nof_items+=game->player[i]->nof_items;
      ltp->nof_says+=game->player[i]->nof_says;
      ltp->secs+=game->player[i]->secs;
      ltp->ttl=(double)ltp->secs/(double)(ltp->nof_deaths+1);
      ltp->effect=ltp_effectiveness(ltp, s1);
      if (ltp->secs)
	ltp->fph=(3600.0/(double)ltp->secs) * ltp->nof_frags;

      if (game->type == GAMETYPE_CTF)
	{
	  ltp->nof_ctfgames++;
	  j=game->player[i]->team+1;
	  if (j>2) j=1;
	  if (game->team_score[game->player[i]->team] > game->team_score[j])
	    ltp->nof_ctfwins++;
	  ltp->ctf_score+=game->player[i]->score;
	  lts->ctf_score+=game->player[i]->score;
	  ltp->ctf_steals+=game->player[i]->ctf_steals;
	  lts->ctf_steals+=game->player[i]->ctf_steals;
	  ltp->ctf_captures+=game->player[i]->ctf_captures;
	  lts->ctf_captures+=game->player[i]->ctf_captures;
	  ltp->ctf_pickups+=game->player[i]->ctf_pickups;
	  lts->ctf_pickups+=game->player[i]->ctf_pickups;
	  ltp->ctf_returns+=game->player[i]->ctf_returns;
	  lts->ctf_returns+=game->player[i]->ctf_returns;
	  ltp->ctf_loss+=game->player[i]->ctf_loss;
	  lts->ctf_loss+=game->player[i]->ctf_loss;
	  ltp->ctf_kills+=game->player[i]->ctf_kills;
	  lts->ctf_kills+=game->player[i]->ctf_kills;
	  ltp->ctf_suic+=game->player[i]->ctf_suic;
	  lts->ctf_suic+=game->player[i]->ctf_suic;
	  ltp->ctf_autoret+=game->player[i]->ctf_autoret;
	  lts->ctf_autoret+=game->player[i]->ctf_autoret;
	}

      for (j=0; j<NOF_WEAPS; j++)
	{
	  ltp->weaps[j]+=game->player[i]->weaps[j];
	  ltp->wtkill[j]+=game->player[i]->wtkill[j];
	  ltp->wsui[j]+=game->player[i]->wsui[j];
	  ltp->wdeath[j]+=game->player[i]->wdeath[j];
	  lts->weaps[j]+=game->player[i]->weaps[j];
	  lts->wtkill[j]+=game->player[i]->wtkill[j];
	  lts->wsui[j]+=game->player[i]->wsui[j];
	}
      for (j=0; j<NOF_ITEMS; j++)
	{
	  ltp->items[j]+=game->player[i]->lst_items[j];
	  lts->items[j]+=game->player[i]->lst_items[j];
	}
    }

  return 1;
}


/* Loading from a file */
t_lts *load_stats(char *fname, hpj_cstack *s1)
{
  long int  size, i, k;
  t_lts    *lts=new_ltStat();
  char      magic[6];

  FILE  *fp=fopen(fname, "rb");
  if (fp==NULL)
    {
      fprintf(stderr, "  load_stats : no lt_stats file found\n");
      return lts;
    }
  fread(magic,5,1,fp);
  if (strncmp(magic,"q3lts",5)!=0)
     {
      fprintf(stderr, "  \"%s\" is not a lt-stats-file\n", fname);
      fclose(fp);
      return lts;
    }
  fread(&loadstats_version,1,1,fp);
  if (loadstats_version!=savestats_version)
    {
      fprintf(stderr, "  \"%s\" has a different version, trying to convert\n",
	      fname);
    }

  size=sizeof(long int);
  fread(&lts->nof_players, size,1,fp);
  fread(&lts->secs, size,1,fp);
  fread(&lts->nof_games, size,1,fp);
  fread(&lts->nof_kills, size,1,fp);
  fread(&lts->nof_tkills, size,1,fp);
  fread(&lts->nof_suicides, size,1,fp);
  fread(&lts->nof_items, size,1,fp);
  fread(&lts->nof_frags, size,1,fp);
  fread(&lts->nof_deaths, size,1,fp);

  if (loadstats_version>'2')
    {
      fread(&lts->nof_ctfgames, size,1,fp);
      fread(&lts->ctf_score, size,1,fp);
      fread(&lts->ctf_steals, size,1,fp);
      fread(&lts->ctf_pickups, size,1,fp);
      fread(&lts->ctf_loss, size,1,fp);
      fread(&lts->ctf_suic, size,1,fp);
      fread(&lts->ctf_autoret, size,1,fp);
      fread(&lts->ctf_captures, size,1,fp);
      fread(&lts->ctf_kills, size,1,fp);
      fread(&lts->ctf_returns, size,1,fp);
    }

  fread(&k,size,1,fp);
  for(i=0; i<k; i++)
    fread(&lts->items[i],size,1,fp);

  fread(&k,size,1,fp);
  for(i=0; i<k; i++)
    {
      fread(&lts->weaps[i],size,1,fp);
      fread(&lts->wsui[i],size,1,fp);
      fread(&lts->wtkill[i],size,1,fp);
    }

  fread(&k,size,1,fp);
  lts->nof_players=k;
  for(i=0; i<k; i++)
    {
      lts->ltplayers[i]=new_ltPlayer();
      load_ltPlayer(lts->ltplayers[i], fp, s1);
    }


  fread(magic,5,1,fp);
  if (strncmp(magic,"q3lte",5)!=0)
    fprintf(stderr, "  \"%s\" was corrupted :-(\n", fname);

  fclose(fp);
  return lts;
}


/* Save the stats to the file */
int save_stats(t_lts *lts, char *fname)
{
  long int size, i, k;

  FILE *fp=fopen(fname, "wb");
  if (fp==NULL)
    {
      fprintf(stderr, "  %s : save_stats : FILE-Error \"%s\"\n",
	      __file__, fname);
      return -1;
    }

  fprintf(fp,"q3lts");
  fprintf(fp,"%c", savestats_version);

  /* write the stats */
  size=sizeof(long int);
  fwrite(&lts->nof_players, size,1,fp);
  fwrite(&lts->secs, size,1,fp);
  fwrite(&lts->nof_games, size,1,fp);
  fwrite(&lts->nof_kills, size,1,fp);
  fwrite(&lts->nof_tkills, size,1,fp);
  fwrite(&lts->nof_suicides, size,1,fp);
  fwrite(&lts->nof_items, size,1,fp);
  fwrite(&lts->nof_frags, size,1,fp);
  fwrite(&lts->nof_deaths, size,1,fp);

  fwrite(&lts->nof_ctfgames, size,1,fp);
  fwrite(&lts->ctf_score, size,1,fp);
  fwrite(&lts->ctf_steals, size,1,fp);
  fwrite(&lts->ctf_pickups, size,1,fp);
  fwrite(&lts->ctf_loss, size,1,fp);
  fwrite(&lts->ctf_suic, size,1,fp);
  fwrite(&lts->ctf_autoret, size,1,fp);
  fwrite(&lts->ctf_captures, size,1,fp);
  fwrite(&lts->ctf_kills, size,1,fp);
  fwrite(&lts->ctf_returns, size,1,fp);

  //fprintf(stderr, "KILLS : %li\n", lts->nof_kills);
  //fprintf(stderr, "FRAGS : %li\n", lts->nof_frags);
  /* now the items */
  k=NOF_ITEMS;
  fwrite(&k,size,1,fp);
  for(i=0; i<k; i++)
    fwrite(&lts->items[i],size,1,fp);

  /* now the weapons ---
     WE JUST WRITE THE NOF_WEAPS ONCE !!!
     Weapons are written as triple !!!
  */
  k=NOF_WEAPS;
  fwrite(&k,size,1,fp);
  for(i=0; i<k; i++)
    {
      fwrite(&lts->weaps[i],size,1,fp);
      fwrite(&lts->wsui[i],size,1,fp);
      fwrite(&lts->wtkill[i],size,1,fp);
    }

  /* finally, write the players */
  k=lts->nof_players;
  fwrite(&k,size,1,fp);
  for(i=0; i<k; i++)
    save_ltPlayer(lts->ltplayers[i], fp);

  fprintf(fp, "q3lte");
  fclose(fp);
  return 1;
}
