#include <stdlib.h>
#include <stdio.h>

#ifdef __WIN32__
#define __NOLIBPNG__
#endif

#ifndef __NOLIBPNG__
#include <png.h>
#include <zlib.h>
#else
#include "png/png.h"
#include "png/zlib.h"
#endif

png_voidp user_error_ptr(){
  fprintf(stderr, "UNKNOWN ERROR !\n");
  exit(-1);
}

static void user_warning_fn(png_structp png_ptr, png_const_charp message){
  fprintf(stderr, "WARNING FILE : %s\n", message);
  exit(-1);
}

static void user_error_fn(png_structp png_ptr, png_const_charp message){
  fprintf(stderr, "ERROR FILE : %s\n", message);
  exit(-1);
}


void write_png(char *fn, unsigned char *data, int dx, int dy,\
	       unsigned char *pal, int compression)
{
  png_structp    png_ptr;
  png_infop      info_ptr;
  png_colorp     p;
  png_byte     **rows;
  int            i;
  FILE          *fp=fopen(fn, "wb");
  if (fp==NULL)
    {
      fprintf(stderr, "  png_write_error\n");
      return;
    }

  p=(png_color *)malloc(sizeof(png_color) * 256);
  for(i=0; i<256; i++)
    {
      p[i].red  =pal[i*3+0];
      p[i].green=pal[i*3+1];
      p[i].blue =pal[i*3+2];
    }
  rows=(png_byte **)malloc(sizeof(png_byte *) * dy);
  for(i=0; i<dy;i++)
    rows[i]=&data[i*dx];

  /* create png*/
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,\
				    (png_voidp)user_error_ptr,
				    user_error_fn, user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "  png_write_error\n");
    return;
  }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr)
    {
      png_destroy_write_struct(&png_ptr,
			       (png_infopp)NULL);
      fprintf(stderr, "  png_write_error\n");
      return;
    }
  setjmp(png_ptr->jmpbuf);

  png_init_io(png_ptr, fp);

  png_set_filter(png_ptr, 0,
		 PNG_FILTER_NONE | PNG_FILTER_SUB |PNG_FILTER_PAETH);
  /* set the zlib compression level */
  switch(compression)
    {
    case 0:
      png_set_compression_level(png_ptr,Z_BEST_SPEED);
      break;
    case 2:
      png_set_compression_level(png_ptr,Z_BEST_COMPRESSION);
      break;
    default:
      png_set_compression_level(png_ptr,Z_DEFAULT_COMPRESSION);
    }
  /* set other zlib parameters */
  png_set_compression_mem_level(png_ptr, 8);
  png_set_compression_strategy(png_ptr,Z_DEFAULT_STRATEGY);
  png_set_compression_window_bits(png_ptr, 15);
  png_set_compression_method(png_ptr, 8);

  
  png_set_IHDR(png_ptr, info_ptr, dx, dy,
	       8, PNG_COLOR_TYPE_PALETTE, PNG_INTERLACE_NONE,
	       PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);

  //fprintf(stderr, "set_palette\n");
  png_set_PLTE(png_ptr, info_ptr, p, 256);

  //fprintf(stderr, "Write_info\n");
  png_write_info(png_ptr, info_ptr);


  //fprintf(stderr, "write_img\n");
  
  //png_write_flush(png_ptr);
  png_write_image(png_ptr, rows);

  png_write_end(png_ptr, info_ptr);

  fclose(fp);
  free(rows);
  free(p);
  png_destroy_write_struct(&png_ptr, &info_ptr);
  //fprintf(stderr, "png written\n");
}











void read_png(char *fn, unsigned char **data, unsigned char *pal,\
	      int *dx, int *dy, int *pe)
{
  png_structp    png_ptr;
  png_infop      info_ptr;
  png_colorp     p;
  png_byte     **rows;
  unsigned char *d;
  int            i, bits, ctype, itype, ftype, compression, palentries;
  unsigned long int       w,h, rowlen;
  FILE          *fp=fopen(fn, "rb");
  if (fp==NULL)
    {
      fprintf(stderr, "  png_read_error\n");
      return;
    }

  /* create png*/
  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,\
				    (png_voidp)user_error_ptr,
				    user_error_fn, user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "  png_read_error\n");
    return;
  }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr)
    {
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL,(png_infopp)NULL);
      fprintf(stderr, "  png_read_error\n");
      return;
    }
  setjmp(png_ptr->jmpbuf);

  png_init_io(png_ptr, fp);
  //png_set_sig_bytes(png_ptr, number);

  //fprintf(stderr, "  read_info\n");
  png_read_info(png_ptr, info_ptr);

  png_get_IHDR(png_ptr, info_ptr, &w, &h, &bits, &ctype, &itype,
	       &compression, &ftype);

  *dx=w; *dy=h;
  //fprintf(stderr, "  %i,%i -> %i bits\n", *dx, *dy, bits);
  if (ctype!=PNG_COLOR_TYPE_PALETTE || bits!=8)
    {
      fprintf(stderr, "  Wrong png-format, must be 8-bit palette\n");
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL,(png_infopp)NULL);
      return;
    }

  rowlen=png_get_rowbytes(png_ptr, info_ptr);
  png_get_PLTE(png_ptr, info_ptr, &p, &palentries);
  *pe=palentries;

  //fprintf(stderr, "  got %i pal-entries from file\n", palentries);

  //fprintf(stderr, "  reading_png\n");
  d=(unsigned char *)malloc(sizeof(png_byte) * rowlen * (*dy));
  rows=(png_byte **)malloc(sizeof(png_byte *) * (*dy));
  for(i=0; i<*dy;i++)
    rows[i]=(png_byte *)(&d[i*rowlen]);

  png_read_image(png_ptr, rows);


  png_read_end(png_ptr, info_ptr);
  memset(pal,0,768);
  for(i=0; i<palentries; i++)
    {
      pal[i*3  ]=p[i].red;
      pal[i*3+1]=p[i].green;
      pal[i*3+2]=p[i].blue;
    }

  png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);

  fclose(fp);
  free(rows);
  //free(p);
  //fprintf(stderr, "png reader closing\n");
  *data=d;
}
