/** Implementation ..

    $Id: hpj_cstack.c,v 1.1 2000/01/17 23:59:28 jacobs Exp $

    $Log: hpj_cstack.c,v $
    Revision 1.1  2000/01/17 23:59:28  jacobs
    Initial revision


*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hpj_cstack.h"

#ifndef __FILE__
#define __FILE__ "hpj_cstack.h"
#endif


hpj_cstack *cstack_new(INT32 initsize)
{
  hpj_cstack *s=(hpj_cstack *)malloc(sizeof(hpj_cstack));
  if (s==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return NULL;
    }
  s->sp=0;
  if (initsize>0) s->capacity=initsize;
  else s->capacity=1;
  s->cont=(CHAR **)malloc(sizeof(CHAR *) * s->capacity);
  if (s->cont==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return NULL;
    }
  return s;
}

hpj_cstack *cstack_clone(hpj_cstack *source)
{
  INT32 i;

  hpj_cstack *s=(hpj_cstack *)malloc(sizeof(hpj_cstack));
  if (s==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return NULL;
    }
  s->sp=source->sp;
  s->capacity=source->capacity;
  s->cont=(CHAR **)malloc(sizeof(CHAR *) * s->capacity);
  if (s->cont==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return NULL;
    }
  for(i=0;i<s->sp; i++)
    s->cont[i]=strdup(source->cont[i]);
  return s;
}

hpj_cstack *cstack_del(hpj_cstack *stack)
{
  INT32 i;

  if (stack==NULL) return NULL;
  for(i=0; i<stack->sp; i++)
    free(stack->cont[i]);
  free(stack->cont);
  free(stack);
  return NULL;
}


CHAR *cstack_pop(hpj_cstack *stack)
{
  stack->sp--;
  if (stack->sp>=0)
    return stack->cont[stack->sp];
  else
    {
      fprintf(stderr, "  %s : Stack empty !\n", __FILE__);
      return NULL;
    }
}

void cstack_push(hpj_cstack *stack, CHAR *mem)
{
  CHAR **tmp;

  if (stack->sp>=stack->capacity-1)
    {
      stack->capacity*=2;
      tmp=(CHAR **)realloc(stack->cont, sizeof(CHAR *) * stack->capacity);
      if (tmp==NULL)
	{
	  fprintf(stderr, "  %s : REALLOC-FAILURE - FATAL\n",__FILE__);
	  exit(-1);
	}
      stack->cont=tmp;
    }
  stack->cont[stack->sp]=strdup(mem);
      if (stack->cont[stack->sp]==NULL)
	{
	  fprintf(stderr, "  %s : STRDUP-FAILURE - FATAL\n",__FILE__);
	  exit(-1);
	}
  stack->sp++;
}

void cstack_push_char(hpj_cstack *stack, CHAR m)
{
  CHAR **tmp;

  if (stack->sp>=stack->capacity-1)
    {
      stack->capacity*=2;
      tmp=(CHAR **)realloc(stack->cont, sizeof(CHAR *) * stack->capacity);
      if (tmp==NULL)
	{
	  fprintf(stderr, "  %s : REALLOC-FAILURE - FATAL\n", __FILE__);
	  exit(-1);
	}
      stack->cont=tmp;
    }
  stack->cont[stack->sp]=(CHAR *)malloc(sizeof(CHAR) * 2);
  if (stack->cont[stack->sp]==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return;
    }
  stack->cont[stack->sp][0]=m;
  stack->cont[stack->sp][1]=0;
  stack->sp++;
}

INT32 cstack_size(hpj_cstack *stack)
{
  INT32 l=sizeof(hpj_cstack);  
  l+=stack->capacity * sizeof(CHAR *);
  return l;
}

void cstack_show(hpj_cstack *stack)
{
  INT32 i;
  printf("cstack uses %i bytes of mem with %i elems on stack, cap = %i\n",
	 cstack_size(stack), stack->sp, stack->capacity);
  for(i=stack->sp-1; i>=0; i--)
    printf(" %02i : |%s|\n", i, stack->cont[i]);
}
