/**  Struct for handling stacks with CHARS.

@author Hans-Peter Jacobs
@version $Id: hpj_cstack.h,v 1.1 2000/01/17 23:59:28 jacobs Exp $

@name hpj_cstack
*/
//@{
#ifndef __HPJ_CHAR_STACK_H__
#define __HPJ_CHAR_STACK_H__


#ifndef INT32
#define INT32 int
#endif

#ifndef CHAR
#define CHAR char
#endif

#ifndef def_boolean
#define def_boolean
typedef enum {false=0, true=1} boolean;
#endif

/** This Macro should be used for casting <b>to</b> hpj_cstack*.*/
#ifndef V2CSTACKP
#define V2CSTACKP(a) (hpj_cstack *)a
#endif


/** @name hpj_cstack - name of the struct for external use*/
//@{
typedef struct
{
  CHAR    **cont;
  INT32     sp, capacity;
}hpj_cstack;
//@}


/** @name Constructors and destructors*/
//@{
/** maxsize sets the initial size (must be > 0) !!
   If size is exceeded, additional 2*"size" bytes are allocated
   @param initsize size of the initial stack, stack is dynamic
   @return pointer to the new struct*/
hpj_cstack *cstack_new(INT32 initsize);

/** Uses an existing cstack to make a copy.
    @param source existing cstack
    @return new created stack, containing the same information like "source"*/
hpj_cstack *cstack_clone(hpj_cstack *source);

/** Frees all used memory, clears the memory up.
    @param stack pointer to the cstack
    @return NULL should be assigned to the old cstack-pointer*/
hpj_cstack *cstack_del(hpj_cstack *stack);
//@}


/** @name Functions for accessing the stack */
//@{
/** Push-Operation for the stack. The char-string will be taken as
    the topmost elem on the stack.
    @param stack hpj_cstack for the push-op
    @param mem pointer to a 0-terminated string*/
void  cstack_push(hpj_cstack *stack, CHAR *mem);

/** Makes a 2-byte char-string out of one byte. So you can add a single
 char and later you "pop" a string.
 @param stack hpj_cstack used for the push-operation
 @param m char used for the new pushed string*/
void  cstack_push_char(hpj_cstack *stack, CHAR m);

/** Takes the topmost element from the stack and returns the pointer to it.
    @param stack pointer to hpj_cstack
    @return pointer to the char-string*/
CHAR *cstack_pop(hpj_cstack *stack);

/** Allows the user to view the topmost element without pop.
    @param stack pointer to hpj_cstack
    @return pointer to the char-string*/
CHAR *cstack_top(hpj_cstack *stack);

/** Returns the memory used by the stack and its struct.
    @param stack pointer to hpj_cstack
    @return size of the used memory (in bytes)*/
INT32 cstack_size(hpj_cstack *stack);

/** This is just for debugging */
void  cstack_show(hpj_cstack *stack);
//@}
//@}
#endif
