/** This is my standard-string-code.
    I programmed it to have an independent code.

    
    @author Hans-Peter Jacobs
    @version $Id: hpj_iarray.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $

	$Id: hpj_iarray.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $
	
	$Log: hpj_iarray.c,v $
	Revision 1.2  2000/01/17 23:59:28  jacobs
	added occurence

	Revision 1.1  2000/01/17 10:27:32  jacobs
	Initial revision

	
*/
#include "hpj_iarray.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifndef __FILE__
#define __FILE__ "hpj_iarray.c"
#endif

/** Just a kind of a constructor.*/
hpj_iarray *iarray_new(INT32 initial_size)
{
  hpj_iarray *i=(hpj_iarray *)malloc(sizeof(hpj_iarray));
  if (i==NULL)
    {
      fprintf(stderr, "  %s : new : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }
  i->length=0;
  if (initial_size<1)
    initial_size=1;
  i->capacity=initial_size;

  i->mem=(INT32 *)malloc(sizeof(INT32) * i->capacity);
  if (i->mem==NULL)
    {
      fprintf(stderr, "  %s : new : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }
  memset(i->mem,0, sizeof(INT32) * i->capacity);

  return i;
}


/** Constructs a new iarray an existing one*/
hpj_iarray *iarray_clone(hpj_iarray *source)
{
  hpj_iarray *i=(hpj_iarray *)malloc(sizeof(hpj_iarray));
  if (i==NULL)
    {
      fprintf(stderr, "  %s : clone : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }

  i->capacity=source->capacity;
  i->length=source->length;
  i->mem=(INT32 *)malloc(sizeof(INT32) * i->capacity);
  if (i->mem==NULL)
    {
      free(i);
      fprintf(stderr, "  %s : clone : MALLOC FAILURE -> FATAL\n", __FILE__);
      return i;
    }
  memcpy(i->mem, source->mem, i->capacity);
  return i;
}


/** Destructor */
hpj_iarray *iarray_del(hpj_iarray *iarray)
{
  if (iarray==NULL)
    {
      fprintf(stderr, "  %s : del : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  //if (iarray->mem!=NULL)
  free(iarray->mem);
  free(iarray);
  return NULL;
}




/*
 *
 *
 simple functions are starting here
 *
 *
 */

/** Returns the length of the stored string */
INT32 iarray_elements(hpj_iarray *i)
{
  if (i!=NULL) return i->length;
  return -1;
}

/** Returns the used memory (in bytes) */
INT32 iarray_size(hpj_iarray *i)
{
  INT32 l=0;
  if (i!=NULL)
    {
      l=sizeof(hpj_iarray);
      l+=i->capacity * sizeof(INT32);
    }
  return l;
}

/** Returns the element at the given position.*/
INT32 iarray_get(hpj_iarray *i, INT32 pos)
{
  if (i==NULL || pos>=i->length)
    {
      fprintf(stderr, "  %s : get : out of bounds-access\n", __FILE__);
      return -1;
    }
  return i->mem[pos];
}

/** Returns the pointer to the internally stored iarray.*/
INT32 *iarray_getIntPtr(hpj_iarray *i)
{
  if (i==NULL)
    {
      fprintf(stderr, "  %s : getIntPtr : NULL-Pointer\n", __FILE__);
      return NULL;
    }
  if (i->length<1 || i->mem==NULL)
    {
      fprintf(stderr, "  %s : getIntPtr : Empy array\n", __FILE__);
      return NULL;
    }
  return i->mem;
}

/** Returns the pointer to a copy of the internally stored iarray.*/
INT32 *iarray_cloneIntPtr(hpj_iarray *i)
{
  INT32 *out;
  if (i==NULL)
    {
      fprintf(stderr, "  %s : cloneIntPtr : NULL-Pointer\n", __FILE__);
      return NULL;
    }
  if (i->length<1 || i->mem==NULL)
    {
      fprintf(stderr, "  %s : cloneIntPtr : Empy array\n", __FILE__);
      return NULL;
    }
  out=(INT32 *)malloc(sizeof(INT32) * i->length);
  if (out==NULL)
    {
      fprintf(stderr, "  %s : cloneIntPtr : MALLOC-Failure\n", __FILE__);
      return NULL;
    }
  memcpy(out, i->mem, sizeof(INT32) * i->length);
  return out;
}


/** Count the commonness of VALUE in IARRAY */
INT32 iarray_occurence(hpj_iarray *i, INT32 v)
{
  INT32 l=0, a;
  if (i==NULL)
    {
      fprintf(stderr, "  %s : occurence : NULL-pointer access\n", __FILE__);
      return 0;
    }
  for (a=0; a<i->length; a++)
    if (i->mem[a]==v) l++;
  return l;
}

/** Returns the highest value within the INT32 - array */
INT32 iarray_getMax(hpj_iarray *i)
{
  INT32 max, a;
  if (i==NULL)
    {
      fprintf(stderr, "  %s : getMax : NULL-pointer access\n", __FILE__);
      return 0;
    }
  if (i->length<1) return 0;
  max=i->mem[0];
  for(a=1; a<i->length; a++)
    if (i->mem[a]>max) max=i->mem[a];
  return max;
}

/** Returns the lowest value within the INT32 - array */
INT32 iarray_getMin(hpj_iarray *i)
{
  INT32 min, a;
  if (i==NULL)
    {
      fprintf(stderr, "  %s : getMin : NULL-pointer access\n", __FILE__);
      return 0;
    }
  if (i->length<1) return 0;
  min=i->mem[0];
  for(a=1; a<i->length; a++)
    if (i->mem[a]<min) min=i->mem[a];
  return min;
}


/** Returns an iarray, which can be used for sorted indexing. */
hpj_iarray *iarray_bubblesortIndex(hpj_iarray *input, boolean a)
{
  INT32       i, t;
  boolean     done;
  hpj_iarray *out;

  if (input==NULL)
    {
      fprintf(stderr, "  %s : bubblesort : NULL-Pointer\n", __FILE__);
      return NULL;
    }

  if (input->length<1)
    {
      fprintf(stderr, "  %s : bubblesort : empty iarray\n", __FILE__);
      return NULL;
    }

  out=iarray_new(2);
  if (out==NULL)
    return NULL;

  /* Sorting index created with 1:1 mapping */
  for (i=0; i<input->length; i++)
    iarray_append(out,i);

  /* If length==1 return NOW */
  if (input->length<2)
    return out;

  /* Bubble-Sort starts here */
  done=false;
  while (!done)
    {
      done=true;
      for(i=1;i<input->length; i++)
	{
	  if (a)
	    {
	      if (input->mem[out->mem[i]] < input->mem[out->mem[i-1]])
		{
		  t            =out->mem[i];
		  out->mem[i]  =out->mem[i-1];
		  out->mem[i-1]=t;
		  done=false;
		}
	    }
	  else
	    {
	      if (input->mem[out->mem[i]] > input->mem[out->mem[i-1]])
		{
		  t            =out->mem[i];
		  out->mem[i]  =out->mem[i-1];
		  out->mem[i-1]=t;
		  done=false;
		}
	    }
	}
    }
  return out;
}









/*
 *
 *
 MODIFYING FUNCTIONS START BELOW
 *
 *
 */
/** Appends a new INT32 to an existing hpj_iarray */
boolean iarray_append(hpj_iarray *i, INT32 v)
{
  INT32 *t=NULL, len;

  if (i==NULL)
    {
      fprintf(stderr, "  %s : append : NULL-pointer access\n", __FILE__);
      return false;
    }
  if (i->length>=(i->capacity-1))
    {
      len=i->capacity*2;
      t=(INT32 *)malloc(sizeof(INT32) * len);
      if (t==NULL)
	{
	  fprintf(stderr, "  %s : append : MALLOC FAILED - FATAL\n", __FILE__);
	  return false;
	}
      memset(t, 0, len*sizeof(INT32));
      memcpy(t, i->mem, sizeof(INT32) * i->length);
      free(i->mem);
      i->mem=t;
      i->capacity=len;
      //fprintf(stderr, "re-alloc to %i for : %i at %i\n", len, v, i->length);
    }
  i->mem[i->length]=v;
  i->length++;
  return true;
}

/** Display the matrix (one line per entry) */
boolean iarray_show(hpj_iarray *i)
{
  INT32 a;

  if (i==NULL)
    {
      fprintf(stderr, "  %s : show : NULL-pointer access\n", __FILE__);
      return false;
    }

  printf("%i elements in iarray, capacity for %i, %i bytes used\n",
	 i->length, i->capacity, iarray_size(i));
  for(a=0; a<i->length; a++)
    printf("%i : %i\n", a, i->mem[a]);
  return true;
}
