/**  Struct for handling an array of INT32.
It contains all necassary informations
to handle UINT32-arrays.
I programmed it to have an independent code.

@author Hans-Peter Jacobs
@version $Id: hpj_iarray.h,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $

@name hpj_iarray
*/
//@{
#ifndef __HPJ_IARRAY_H__
#define __HPJ_IARRAY_H__

#ifndef INT32
#define INT32 int
#endif

#ifndef UINT8
#define UINT8 unsigned char
#endif

/** This Macro should be used for casting from a VOIDP <b>to</b> hpj_iarray*.
    So you can keep an parray, pstack, etc. with iarrays inside.*/
#ifndef V2IARRAYP
#define V2IARRAYP(a) ((hpj_iarray *)a)
#endif

#ifndef def_boolean
#define def_boolean
typedef enum {false, true} boolean;
#endif


/** @name hpj_iarray - name of the struct for external use*/
//@{
typedef struct
{
  INT32  *mem;
  INT32   length;
  INT32   capacity;
}hpj_iarray; 
//@}











/** @name Constructors and destructors*/
//@{
/** Creates a new struct. There will be space for "initial_size" elemets.
    If this bound is reached "initial_size" * 2 bytes will be allocated.. etc.
    @param initial_size Sets the initial memory allocated for elems.
    @return pointer to the new created struct*/
hpj_iarray *iarray_new(INT32 initial_size);


/** Creates a new hpj_iarray from an existing.
    @param source existing hpj_iarray
    @return newly created hpj_iarray, copied from the source
*/
hpj_iarray *iarray_clone(hpj_iarray *source);

/** Releases all memory allocated for the hpj_iarray.
    It returns NULL, so the calling function can assign
    the result to the old "hpj_iarray"-pointer.
    @param hpj_iarray pointer to the iarray-structure
    @return NULL
*/
hpj_iarray *iarray_del(hpj_iarray *iarray);
//@}


/** This functions provide informations about the iarray.
    @name Functions (simple)*/
//@{
/** Returns the number of elements stored within the iarray.
    @param iarray pointer to the desired hpj_iarray-struct
    @return number of elements within the iarray*/
INT32 iarray_elements(hpj_iarray *iarray);

/** Returns the length of the iarray-memory-block.
    Please note - this differs from the number of elements stored !!!
    @param iarray pointer to the hpj_iarray-struct
    @return memory used by the struct and the allocated memory-block*/
INT32 iarray_size(hpj_iarray *iarray);

/** Returns the element at the given position.
    @param iarray pointer to the struct
    @param pos position of the desired element
    @return value at the position*/
INT32 iarray_get(hpj_iarray *iarray, INT32 pos);

/** Returns the pointer to the internally stored iarray.
    <b>SHOULD JUST BE USED FOR DIRECT (FAST) READING!!!</b>
    @param iarray pointer to the existing hpj_iarray
    @return pointer to the integer-array*/
INT32 *iarray_getIntPtr(hpj_iarray *i);

/** Returns the a pointer to a copy of the internally stored iarray.
    This array must be freed manually !!!!<br>
    The length of the array is : <i>sizeof(INT32) * iarray->length</i>.
    @param iarray pointer to the existing hpj_iarray
    @return pointer to a new INT32-array*/
INT32 *iarray_cloneIntPtr(hpj_iarray *iarray);

/** This counts how often the given INT32-value appears in the array.
    @param iarray pointer to the array which will be searched
    @param value INT32-value to search for
    @return number of occurence of <b>value</b> in <b>iarray</b>*/
INT32 iarray_occurence(hpj_iarray *iarray, INT32 value);

/** Returns the maximal value within the iarray.
    @param iarray pointer to the array
    @return maximal INT32-value found in the array*/
INT32 iarray_getMax(hpj_iarray *iarray);

/** Returns the minimal value within the iarray.
    @param iarray pointer to the array
    @return minimal INT32-value found in the array*/
INT32 iarray_getMin(hpj_iarray *iarray);

/** This function sorts the existing hpj_iarray, returning an hpj_iarray
    with the correct indizes. The existing iarray will not be affected,
    the indexing (returned) hpj_iarray contains the right order.
    @param iarray pointer to the unsorted hpj_iarray
    @param descend if true, iarray will be sorted descending, ascending else
    @return a pointer to an idexing iarray*/
hpj_iarray *iarray_bubblesortIndex(hpj_iarray *iarray, boolean descend);
//@}


/** The following functions modify the data within the array.
    @name Functions*/
//@{
/** This adds a new INT32 to the existing memory-block.
  @param iarray existing iarray
  @param value new INT32 value
  @return boolean value (true/false) for success
*/
boolean iarray_append(hpj_iarray *iarray, INT32 value);

/** Displays the content of the iarray on stdout.
    @param iarray hpj_iarray for output
    @return true or false for success*/
boolean iarray_show(hpj_iarray *iarray);
//@}

#endif
//@}

