/*
  This is a basic collection of functions for the
  PAL-struct.

  $Id$

  $Log$

*/
#ifndef __file__
#define __file__ "hpj_image.c"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "hpj_image.h"
#include "hpj_tfile.h"


/*
 *
 *
 Creations and settings
 *
 *
 */
hpj_image *image_new()
{
  hpj_image *i=(hpj_image *)malloc(sizeof(hpj_image));
  memset(i, 0, sizeof(hpj_image));

  i->palmem=(UCHAR *)malloc(768);
  memset(i->palmem,0,768);
  return i;
}


hpj_image *image_del(hpj_image *i)
{
  if (i==NULL) return NULL;
  if (i->palmem) free(i->palmem);
  if (i->mem) free(i->mem);
  if (i->filename) free(i->filename);
  free(i);
  return NULL;
}

INT32 image_createData(hpj_image *i, int dx, int dy)
{
  if (i==NULL)
    {fprintf(stderr, "  %s : createData : NULL-Ptr\n", __file__);
    return 0;}
  if (i->mem!=NULL) free(i->mem);
  i->mem=(UCHAR *)malloc(sizeof(CHAR) * dx * dy);
  memset(i->mem, 0, sizeof(CHAR) * dx * dy);
  i->width=dx;
  i->height=dy;
  return 1;
}

INT32 image_setFilename(hpj_image *i, char *fname)
{
  if (i==NULL)
    {fprintf(stderr, "  %s : setFilename : NULL-Ptr\n", __file__);
    return 0;}
  if (i->filename) free(i->filename);

  if (fname!=NULL)
    i->filename=strdup(fname);
  else i->filename=fname;
  return 1;
}




/*
 *
 *
 ACCESSING THE DATAS
 *
 *
 */
CHAR *image_copyData(hpj_image *i)
{
  CHAR *c=NULL;
  if (i==NULL)
    {fprintf(stderr, "  %s : createData : NULL-Ptr\n", __file__);
    return NULL;}
  c=(CHAR *)malloc(sizeof(CHAR) * i->width * i->height);
  memcpy(c, i->mem, sizeof(CHAR) * i->width * i->height);
  return c;
}


/*
 *
 *
 PALETTE-Functions starts here
 *
 *
 */
int image_loadPal(hpj_image *i, char *filename)
{
  hpj_tfile *f;
  hpj_string *s;
  int palcounter=0, c1, c2, c3;
  unsigned char  *m;

  if (i==NULL)
    {
      fprintf(stderr, "  %s : loadPal : NULL-Ptr\n", __file__);
      return -1;
    }

  f=tfile_new(filename);
  if (tfile_open(f)==false)
    return -1;

  m=i->palmem;
  //fprintf(stderr, "  loading palette from \"%s\"\n", filename);
  while(!tfile_eof(f))
    {
      s=tfile_getLine(f);
      if (string_length(s)<1)
	{string_del(s);continue;}
      if (string_char(s,0)=='#')
	{string_del(s);continue;}
      //fprintf(stderr, "%s\n", string_charptr(s));
      if (sscanf(string_charptr(s), "%i %i %i", &c1, &c2, &c3)==3)
	{
	  //fprintf(stderr, "%i %i %i\n", c1, c2, c3);
	  m[palcounter*3  ]=(unsigned char)c1;
	  m[palcounter*3+1]=(unsigned char)c2;
	  m[palcounter*3+2]=(unsigned char)c3;
	  palcounter++;
	}
      s=string_del(s);
    }
  fprintf(stderr, "  loaded %i palette-entries from %s\n",\
	  palcounter, filename);
  i->palnums=palcounter;
  tfile_del(f);
  return 1;
}


void image_getPal(hpj_image *i, hpj_image *s)
{
  if (s==NULL || i==NULL || i->palmem==NULL || s->palmem==NULL)
    {
      fprintf(stderr, "  %s : getPal : NULL-Ptr\n", __file__);
      return;
    }
  memcpy(i->palmem, s->palmem, 768);
}


char *image_palRefToRRGGBB(hpj_image *i, int index)
{
  if (index<0 || index>767)
    strcpy(i->rrggbb, "000000");
  else
    {
      sprintf(i->rrggbb, "%02x%02x%02x", i->palmem[index*3+0],\
	      i->palmem[index*3+1], i->palmem[index*3+2]);
    }
  return i->rrggbb;
}



/*
 *
 *
 *
 STUFF FOR LOADING PNG COMES HERE
 *
 *
 *
 */
