/** This is my standard-string-code.
    I programmed it to have an independent code.

    
    @author Hans-Peter Jacobs
    @version $Id: hpj_parray.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $

	$Id: hpj_parray.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $
	
	$Log: hpj_parray.c,v $
	Revision 1.2  2000/01/17 23:59:28  jacobs
	added occurence

	Revision 1.1  2000/01/17 10:27:32  jacobs
	Initial revision

	
*/
#include "hpj_parray.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifndef __FILE__
#define __FILE__ "hpj_parray.c"
#endif

/** Just a kind of a constructor.*/
hpj_parray *parray_new(INT32 initial_size)
{
  hpj_parray *p=(hpj_parray *)malloc(sizeof(hpj_parray));
  if (p==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }
  p->length=0;
  p->capacity=initial_size;
  if (p->capacity<1) p->capacity=1;

  p->mem=(VOIDP *)malloc(sizeof(VOIDP) * p->capacity);
  if (p->mem==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }
  memset(p->mem,0, sizeof(VOIDP) * p->capacity);

  return p;
}


/** Constructs a new iarray an existing one*/
hpj_parray *parray_clone(hpj_parray *source)
{
  hpj_parray *p=(hpj_parray *)malloc(sizeof(hpj_parray));
  if (p==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILURE - FATAL\n", __FILE__);
      return NULL;
    }

  p->capacity=source->capacity;
  p->length=source->length;
  p->mem=(VOIDP *)malloc(sizeof(VOIDP) * p->capacity);
  if (p->mem==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILURE -> FATAL\n", __FILE__);
      return p;
    }
  memcpy(p->mem, source->mem, p->capacity);
  return p;
}


/** Destructor */
hpj_parray *parray_del(hpj_parray *parray)
{
  if (parray==NULL)
    {
      fprintf(stderr, "  %s : NULL-ptr\n", __FILE__);
      return NULL;
    }
  if (parray->mem!=NULL) free(parray->mem);
  free(parray);
  return NULL;
}




/*
 *
 *
 simple functions are starting here
 *
 *
 */

/** Returns the length of the stored string */
INT32 parray_elements(hpj_parray *p)
{
  if (p!=NULL) return p->length;
  return -1;
}

/** Returns the used memory (in bytes) */
INT32 parray_size(hpj_parray *p)
{
  INT32 l=0;
  if (p!=NULL)
    {
      l=sizeof(hpj_parray);
      l+=p->capacity * sizeof(VOIDP);
    }
  return l;
}

/** Returns the element at the given position.*/
VOIDP parray_get(hpj_parray *p, INT32 pos)
{
  if (p==NULL || pos>=p->length)
    {
      fprintf(stderr, "  %s : get : bounds-error\n", __FILE__);
      return NULL;
    }
  return p->mem[pos];
}

/** SETS the element at the given position.*/
boolean parray_set(hpj_parray *p, VOIDP pointer, INT32 pos)
{
  INT32  l;
  VOIDP *t;

  if (p==NULL)
    {
      fprintf(stderr, "  %s : set : bounds-error\n", __FILE__);
      return false;
    }
  if (pos>=p->capacity)
    {
      l=p->capacity;
      while(l<=pos)
	l*=2;
      t=(VOIDP *)malloc(sizeof(VOIDP) * l);
      if (t==NULL)
	{
	  fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
	  return false;
	}
      memset(t, 0, l*sizeof(VOIDP));
      memcpy(t, p->mem, sizeof(VOIDP) * p->length);
      free(p->mem);
      p->mem=t;
      p->capacity=l;
      p->length=pos;
    }
  p->mem[pos]=pointer;
  return true;
}

/** SETS the element at the given position.*/
boolean parray_safe_set(hpj_parray *p, VOIDP pointer, INT32 pos)
{
  if (p==NULL)
    {
      fprintf(stderr, "  %s : safe_set : bounds-error\n", __FILE__);
      return false;
    }
  if (pos<p->length)
    {
      p->mem[pos]=pointer;
      return true;
    }
  fprintf(stderr, "  %s : safe_set : bounds-error\n", __FILE__);
  return false;
}

/** Extends the size of the array */
boolean parray_expand(hpj_parray *p, INT32 elements)
{
  VOIDP *t=NULL;
  INT32  l;

  if (elements<p->capacity)
    {
      p->length=elements;
      return true;
    }

  l=p->capacity;
  while(l<=elements)
    l*=2;
  t=(VOIDP *)malloc(sizeof(VOIDP) * l);
  if (t==NULL)
    {
      fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
      return false;
    }
  memset(t, 0, l*sizeof(VOIDP));
  memcpy(t, p->mem, sizeof(VOIDP) * p->length);
  free(p->mem);
  p->mem=t;
  p->capacity=l;
  p->length=elements;
  return true;
}


/** Appends a new VOIDP to an existing hpj_iarray */
boolean parray_append(hpj_parray *p, VOIDP v)
{
  VOIDP *t=NULL;
  INT32  len;

  if (p==NULL)
    {
      fprintf(stderr, "  %s : NULL-pointer access\n", __FILE__);
      return false;
    }
  if (p->length>=(p->capacity-1))
    {
      len=p->capacity*2;
      t=(VOIDP *)malloc(sizeof(VOIDP) * len);
      if (t==NULL)
	{
	  fprintf(stderr, "  %s : MALLOC FAILED - FATAL\n", __FILE__);
	  return false;
	}
      memset(t, 0, len*sizeof(VOIDP));
      memcpy(t, p->mem, sizeof(VOIDP) * p->length);
      free(p->mem);
      p->mem=t;
      p->capacity=len;
      //fprintf(stderr, "re-alloc to %i for : %i at %i\n", len, v, i->length);
    }
  p->mem[p->length]=v;
  p->length++;
  return true;
}

/** Display the matrix*/
boolean parray_show(hpj_parray *p)
{
  //INT32 a;

  if (p==NULL) return false;
  printf("%i elements in parray, capacity for %i, %i bytes used\n",
	 p->length, p->capacity, parray_size(p));
  /*
  for(a=0; a<p->length; a++)
    printf("%i : %x\n", a, p->mem[a]);
  */
  return true;
}
