/** This is my standard-string-code.
    I programmed it to have an independent code.
    
    @author Hans-Peter Jacobs
    @version $Id: hpj_string.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $
    
    $Id: hpj_string.c,v 1.2 2000/01/17 23:59:28 jacobs Exp jacobs $
    
    $Log: hpj_string.c,v $
    Revision 1.2  2000/01/17 23:59:28  jacobs
    *** empty log message ***

    Revision 1.1  2000/01/17 10:27:32  jacobs
    Initial revision
	
*/
#include "hpj_string.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifndef __FILE__
#define __FILE__ "hpj_string.c"
#endif

/** Just a kind of a constructor.*/
hpj_string *string_new()
{
  hpj_string *s=(hpj_string *)malloc(sizeof(hpj_string));
  if (s==NULL)
    {
      fprintf(stderr, "  %s : new : MALLOC FAILURE\n", __FILE__);
      return NULL;
    }
  s->length=0;
  s->mem=NULL;
  return s;
}


/** Constructs a new string from an existing CHAR-pointer*/
hpj_string *string_new_char(CHAR *source)
{
  hpj_string *s;
  if (source==NULL)
    {
      fprintf(stderr, "  %s : new_char : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  
  s=string_new();
  s->mem=strdup(source);
  s->length=strlen(source);
  return s;
}


/** Creates a new hpj_String with a pointer to the existing char-string.
There are no copies used within this function !*/
hpj_string *string_new_ref_char(CHAR *source)
{
  hpj_string *s=string_new();
  if (source==NULL) return s;
  s->mem=source;
  s->length=strlen(source);
  return s;
}

/** Makes a copy of the existing hpj_string.*/
hpj_string *string_clone(hpj_string *source)
{
  hpj_string *s=string_new();
  s->mem=strdup(string_charptr(source));
  s->length=strlen(s->mem);
  return s;
}


/** Destructor */
hpj_string *string_del(hpj_string *string)
{
  if (string==NULL)
    {
      fprintf(stderr, "  %s : del : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  if (string->mem!=NULL) free(string->mem);
  free(string);
  return NULL;
}




/*
 *
 *
 simple functions are starting here
 *
 *
 */
/** Returns the value of the CHAR at the given position */
CHAR string_char(hpj_string *s, INT32 pos)
{
  if (s==NULL)
    {
      fprintf(stderr, "  %s : char : NULL-Ptr\n", __FILE__);
      return 0;
    }
  if (s->length<=pos)
    {
      fprintf(stderr, "  %s : char : out of bounds\n", __FILE__);
      return 0;
    }
  return s->mem[pos];
}

/** Counts the occurence of value in string */
INT32 string_count_char(hpj_string *s, CHAR v)
{
  INT32 i, sum;
  if (s==NULL)
    {
      fprintf(stderr, "  %s : char : NULL-Ptr\n", __FILE__);
      return 0;
    }
  sum=0;
  for(i=0; i<s->length; i++)
    if (s->mem[i]==v) sum++;
  return sum;
}

/** Return the char-ptr */
CHAR *string_charptr(hpj_string *s)
{
  if (s!=NULL) return s->mem;
  return NULL;
}

/** Returns a new char-ptr, containing the string */
CHAR *string_clone_charptr(hpj_string *s)
{
  CHAR *c;
  if (s==NULL)
    {
      fprintf(stderr, "  %s : clone_charptr : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  c=(CHAR *)malloc(sizeof(CHAR) * (s->length+1));
  if (c==NULL)
    {
      fprintf(stderr, "  %s : clone_charptr : MALLOC_FAILURE\n", __FILE__);
      return NULL;
    }
  memset(c, 0, sizeof(CHAR) * (s->length+1));
  memcpy(c, s->mem, sizeof(CHAR) * s->length);
  return c;
}

/** Compare - function */
INT32 string_compare_charptr(hpj_string *s, CHAR *c, boolean ci)
{
  if (s==NULL)
    {
      fprintf(stderr, "  %s : compare_charptr : NULL-Ptr\n", __FILE__);
      return 0;
    }
  if (ci==false) return strcmp(s->mem, c);
  else return strcasecmp(s->mem, c);
}


/** Returns the length of the stored string */
INT32 string_length(hpj_string *s)
{
  if (s!=NULL) return s->length;
  return -1;
}

/** Returns the used memory (in bytes) */
INT32 string_size(hpj_string *s)
{
  INT32 l=0;
  if (s!=NULL)
    {
      l=sizeof(hpj_string);
      l+=s->length;
    }
  return l;
}

/** Look for a String in String. */
hpj_string *string_getSiS(hpj_string *s, CHAR d)
{
  INT32       start, end;
  hpj_string *s1=NULL;

  if (s==NULL || s->mem==NULL)
    {
      fprintf(stderr, "  %s : getSiS : NULL-Ptr\n", __FILE__);
      return false;
    }
  start=0;
  while(s->mem[start]!=d && start<s->length)
    start++;
  if (start>=s->length) return NULL;
  end=++start;
  while(s->mem[end]!=d && end<s->length)
    end++;
  if (end>=s->length || end-start-1<1) return NULL;
  s1=string_new();
  s1->mem=(CHAR *)malloc(sizeof(CHAR) * (1+end-start));
  s1->length=end-start;
  memset(s1->mem, 0, s1->length+1);
  memcpy(s1->mem, &s->mem[start],s1->length);
  return s1;
}


/** Returns a copy of the string (left start to the delimiter) */
hpj_string *string_token_left(hpj_string *s, CHAR d)
{
  hpj_string *s1;
  INT32 pos;

  if (s==NULL)
    {
      fprintf(stderr, "  %s : token_left : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  pos=0;
  while(s->mem[pos]!=d && pos<s->length)
    pos++;
  if (pos>=s->length) return NULL;
  s1=string_new();
  s1->mem=(CHAR *)malloc(sizeof(CHAR) * (pos+1));
  if (s1->mem==NULL)
    {
      fprintf(stderr, "  %s : token_left : MALLOC-FAILURE\n", __FILE__);
      string_del(s1);
      return NULL;
    }
  memcpy(s1->mem, s->mem, pos);
  s1->mem[pos]=0;
  s1->length=pos;
  return s1;
}

/** Returns a copy of the string (delimiter to right end) */
hpj_string *string_token_right(hpj_string *s, CHAR d)
{
  hpj_string *s1;
  INT32 pos, len;

  if (s==NULL)
    {
      fprintf(stderr, "  %s : token_right : NULL-Ptr\n", __FILE__);
      return NULL;
    }
  pos=0;
  while(s->mem[pos]!=d && pos<s->length)
    pos++;
  if (pos>=s->length) return NULL;
  s1=string_new();
  len=s->length-pos-1;
  s1->mem=(CHAR *)malloc(sizeof(CHAR) * (len+1));
  if (s1->mem==NULL)
    {
      fprintf(stderr, "  %s : token_right : MALLOC-FAILURE\n", __FILE__);
      string_del(s1);
      return NULL;
    }
  memcpy(s1->mem, &s->mem[pos+1], len);
  s1->mem[len]=0;
  s1->length=len;
  return s1;
}

/* Displays a string */
boolean string_show(hpj_string *s)
{
  if (s==NULL)
    {
      fprintf(stderr, "  %s : show = NULL-Ptr\n", __FILE__);
      return false;
    }
  printf(" hpj_string: %i bytes used, %i length\n content   : ->%s<-\n",\
	 string_size(s), s->length, s->mem);
  return true;
}







/*
 *
 *
 MODIFYING Functions are comming below
 *
 *
 *
 */
/** Appends a string from hpj_string to another hpj_string */
boolean string_append(hpj_string *s, hpj_string *append)
{
  INT32 len;
  CHAR *t;

  if (s==NULL)
    {
      fprintf(stderr, "  %s : append : NULL-Ptr\n", __FILE__);
      return false;
    }
  len=strlen(s->mem) + strlen(append->mem) + 1;
  t=(CHAR *)malloc(sizeof(CHAR)*len);
  if (t==NULL)
    {
      fprintf(stderr, "  %s : append : REALLOC FAILURE\n", __FILE__);
      return false;
    }
  memcpy(t,s->mem,strlen(s->mem));
  memcpy(&t[strlen(s->mem)],append->mem,strlen(append->mem));
  t[len]=0;
  free(s->mem);
  s->mem=t;
  s->length=len;
  return true;
}

/** Appends a char-string to an existing hpj_string*/
boolean string_append_char(hpj_string *s, CHAR *append)
{
  INT32  len;
  CHAR  *t;

  if (s==NULL)
    {
      fprintf(stderr, "  %s : append_char : NULL-Ptr\n", __FILE__);
      return false;
    }  
  len=s->length + strlen(append) + 1;
  t=(CHAR *)malloc(sizeof(CHAR)*len);
  if (t==NULL)
    {
      fprintf(stderr, "  %s : append_char : REALLOC FAILURE\n", __FILE__);
      return false;
    }
  memset(t,0,len);
  memcpy(t,s->mem,s->length);
  memcpy(&t[s->length],append,strlen(append));
  free(s->mem);
  s->mem=t;
  s->length=len-1;
  return true;
}


/** Appends a char to an existing hpj_string*/
boolean string_append_single_char(hpj_string *s, CHAR append)
{
  INT32  len;
  CHAR  *t;

  if (s==NULL)
    {
      fprintf(stderr, "  %s : append_single_char : NULL-Ptr\n", __FILE__);
      return false;
    }  
  len=s->length + 2;
  t=(CHAR *)malloc(sizeof(CHAR)*len);
  if (t==NULL)
    {
      fprintf(stderr, "  %s : append_single_char : REALLOC FAILURE\n",\
	      __FILE__);
      return false;
    }
  memset(t,0,len);
  memcpy(t,s->mem,s->length);
  t[s->length]=append;
  if (s->mem!=NULL) free(s->mem);
  s->mem=t;
  s->length=len-1;
  return true;
}


/** Shifts the string to the left, good for parsing */
boolean string_shiftLeft(hpj_string *s, INT32 n)
{
  CHAR  *t;
  INT32  newlen;

  if (s==NULL || s->mem==NULL)
    {
      fprintf(stderr, "  %s : shiftLeft : NULL-Ptr\n", __FILE__);
      return false;
    }

  newlen=s->length-n;
  if (newlen<1)
    {
      s->length=0;
      free(s->mem);
      s->mem=NULL;
      return false;
    }
  t=(CHAR *)malloc(sizeof(CHAR) * (newlen+1));
  if (t==NULL)
    {
      fprintf(stderr, "  %s : shiftLeft : MALLOC-FAILURE\n", __FILE__);
      return false;
    }
  memset(t,0, sizeof(CHAR) * (newlen+1));
  memcpy(t, &s->mem[n], newlen);
  free(s->mem);
  s->mem=t;
  s->length=newlen;
  return true;
}
