/**  This is a very basic string-handler.
It contains all necassary informations
to handle a character-string.
I programmed it to have an independent code.

@author Hans-Peter Jacobs
@version $Id: hpj_string.h,v 1.3 2000/01/17 23:59:28 jacobs Exp jacobs $


@name hpj_string
*/
//@{
#ifndef __HPJ_STRING_H__
#define __HPJ_STRING_H__

#ifndef INT32
#define INT32 int
#endif

#ifndef UINT8
#define UINT8 unsigned char
#endif

#ifndef CHAR
#define CHAR char
#endif

/** This Macro should be used for casting from a VOIDP <b>to</b> hpj_string*.*/
#ifndef V2STRP
#define V2STRP(a) ((hpj_string *)a)
#endif

#ifndef def_boolean
#define def_boolean
typedef enum {false, true} boolean;
#endif


/** @name hpj_string - name of the struct for external use*/
//@{
typedef struct hpj_string
{
  CHAR    *mem;
  INT32   length;
}hpj_string; 
//@}











/** @name Constructors and destructors*/
//@{
/** String new creates an empty string (0 bytes long).*/
hpj_string *string_new();

/** Creates a new hpj_string and copies the source.
    @param source character-pointer to a 0-terminated string
    @return newly created hpj_string, contains a copy of the source-string*/
hpj_string *string_new_char(CHAR *source);

/** Creates a new hpj_string and uses the existing (given) pointer
    as reference to the	existing string.<br>
    DO NOT TOUCH THE "source" external after this operation !!
    @param source pointer to an existing 0-terminated string
    @return newly created hpj_string*/
hpj_string *string_new_ref_char(CHAR *source);

/** Creates a new hpj_string from an existing.
    @param source existing hpj_string
    @return newly created hpj_string, copied from the source*/
hpj_string *string_clone(hpj_string *source);

/** Releases all memory allocated for the hpj_string.
    It returns NULL, so the calling function can assign
    the result to the old "hpj_string"-pointer.
    @param string pointer to the string-structure
    @return NULL*/
hpj_string *string_del(hpj_string *string);
//@}








/** Operation which can be used for getting  infos or pointers to the string.
    This Ops do not modify the original string !!
    @name Functions (not modifying) */
//@{
/** Returns the CHAR at the given position.
    If <b>pos</b>-index is ut of bounds the return-value is set to 0.
    @param string pointer to the hpj_string-struct
    @param pos position within the string
    @return CHAR-value at the position <b>pos</b>, 0 for out-of-bounds*/
CHAR string_char(hpj_string *string, INT32 pos);

/** Counts the occurence of the given char
    @param string pointer to the hpj_string-struct
    @param value CHAR-value to look for in the string
    @return how often "value" was found in "string"*/
INT32 string_count_char(hpj_string *string, CHAR value);

/** Returns a char-ptr for low-level outputs.
    @param string hpj_string for output
    @return char-ptr to raw-string*/
CHAR *string_charptr(hpj_string *string);

/** Returns the char-ptr to a new memory block, containing a copy of
    the string. Please note, that "string", the pointer to the hpj_string
    is still valid.
    @param string pointer to the hpj_string-struct
    @return CHAR-pointer to a new memory-block*/
CHAR *string_clone_charptr(hpj_string *s);

/** Compares a char - pointer with a hpj_string.
    @param string pointer to hpj_string-struct
    @param charptr pointer to the 0-terminated char-string
    @param ci if set to <b>true</b> the strings will be compared
    case-insensitive, else they will be compared case-sensitive
    @return 0 is equal, -1 if string is smaller, 1 else*/
INT32 string_compare_charptr(hpj_string *string, CHAR *charptr, boolean ci);

/** Returns the length of the string. Although the struct is very simple
    you never should make use of the direct access.
    @param string pointer to the hpj_string
    @return Length of the string*/
INT32 string_length(hpj_string *string);

/** Returns the size of the memory used by the struct and the allocated
    memory.<br><b>Warning</b> - this differs from the length of the string !!!
    @param string pointer to the hpj_string
    @return Length of the string*/
INT32 string_size(hpj_string *string);

/** This takes out a string out of the string. StringInString is it
    called. You must define a delimiter and the functions returns a new
    string which is bounded with this delimiter. Together with the
    <b>string_shiftLeft</b>-functions a string can be parsed.
    @param string pointer to the hpj_string-struct.
    @param delimiter defines the "borders" (delimiters) of the string
    @return pointer to a new hpj_string-struct (copy)*/
hpj_string *string_getSiS(hpj_string *string, CHAR delimiter);

/** Searchs for a delimiter, returns a copy of the string from position 0 to
    that token. The delimiter wil be excluded from the copy.
    @param string pointer to the existing hpj_string-struct
    @param delimiter this CHAR is used for splitting the string into tokens
    @return pointer to a new string or NULL if the search failed*/
hpj_string *string_token_left(hpj_string *string, CHAR delimiter);

/** Searchs for a delimiter, returns a copy of the string from the first
    occurence of the delimiter (seen from left) to the end of the string.
    The delimiter is not in the copy.
    @param string pointer to the existing hpj_string-struct
    @param delimiter this CHAR is used for splitting the string into tokens
    @return pointer to a new string or NULL if the search failed*/
hpj_string *string_token_right(hpj_string *string, CHAR delimiter);

/** Displays some informations about the string and the string itself
    @param string hpj_string for output
    @return true or false (for NULL-Ptr)*/
boolean string_show(hpj_string *s);
//@}







/** This Operations can be used to modify the existing string !!!
    @name Functions */
//@{
/** This adds an existing string to the given hpj_string.
  The string for appending will be copied and exists untouched in
  the append-hpj_string.
  @param string existing hpj_string
  @param append string for appending
  @return boolean value (true/false) for success*/
boolean string_append(hpj_string *string, hpj_string *append);


/** This adds an existing char-string to the given hpj_string.
The string for appending will be copied and not be touched.
@param string existing hpj_string
@param append string for appending
@return boolean value (true/false) for success*/
boolean string_append_char(hpj_string *string, CHAR *append);

/** This adds a single character to the given hpj_string.
The string will be expanded.
@param string existing hpj_string
@param append character for appending
@return boolean value (true/false) for success*/
boolean string_append_single_char(hpj_string *string, CHAR append);

/** This shifts the string to the left -> the string will be cut !
    The first <b>number</b> characters will be erased, the string is shorter
    after this operation.
    @param string points to the hpj_string-struct
    @param number number of shift-left operations to be executed
    @return true if succes, false if string to short, etc..*/
boolean string_shiftLeft(hpj_string *string, INT32 number);
//@}

#endif
//@}

