/**  Struct for handling an FILE as text-file.

@author Hans-Peter Jacobs
@version $Id$

@name hpj_tfile
*/
//@{
#ifndef __HPJ_TFILE_H__
#define __HPJ_TFILE_H__

#include "hpj_string.h"

#ifndef def_boolean
#define def_boolean
typedef enum {false, true} boolean;
#endif

#ifndef CHAR
#define CHAR unsigned char
#endif

#ifndef INT32
#define INT32 int
#endif

/** @name hpj_tfile - name of the struct for external use*/
//@{
typedef struct
{
  FILE   *fp;
  INT32   length;
  CHAR    a;
  CHAR   *filename;
}hpj_tfile;
//@}

/** @name Constructors and destructors*/
//@{
/** Creates a new struct. The file will not be opened.
    @param filename name for the file to be opened
    @return pointer to the new tfile-struct*/
hpj_tfile *tfile_new(CHAR *filename);

/** Releases any used memory. If file is still open, it will be closed.
    @param tfile pointer to the corresponding file-struct.
    @return NULL*/
hpj_tfile *tfile_del(hpj_tfile *tfile);
//@}

/** Operations for opening, closing, reading and writing a file.
    @name Functions*/
//@{
/** This opens a file for reading.
    @param f pointer to the file-struct
    @return true for success, false else*/
boolean tfile_open(hpj_tfile *tfile);

/** This closes the opened file.
    @param f pointer to the file-struct
    @return true for success, false else*/
boolean tfile_close(hpj_tfile *tfile);

/** Returns <b>true</b> if the file is at the end, <b>false</b> else.
    @param tfile pointer to the file-struct
    @return true for EOF, false else*/
boolean tfile_eof(hpj_tfile *tfile);

/** This gets a line, puts the contents into a string-struct.
    @param tfile pointer to the tfile-struct
    @return pointer to the hpj_string readed from the file*/
hpj_string *tfile_getLine(hpj_tfile *tfile);
//@}

#endif
//@}
